@@ -1,10 +1,9 @@
 # --
-# Kernel/GenericInterface/Transport/HTTP/Test.pm - GenericInterface network transport interface for testing
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::GenericInterface::Transport::HTTP::Test;
@@ -16,9 +15,10 @@ use HTTP::Request::Common;
 use LWP::UserAgent;
 use LWP::Protocol;
 
-use Kernel::System::Web::Request;
+# prevent 'Used once' warning for Kernel::OM
+use Kernel::System::ObjectManager;
 
-use vars qw(@ISA);
+our $ObjectManagerDisabled = 1;
 
 =head1 NAME
 
@@ -35,26 +35,9 @@ Kernel::GenericInterface::Transport::Test - GenericInterface network transport i
 usually, you want to create an instance of this
 by using Kernel::GenericInterface::Transport->new();
 
-    use Kernel::Config;
-    use Kernel::System::Encode;
-    use Kernel::System::Log;
     use Kernel::GenericInterface::Transport;
 
-    my $ConfigObject = Kernel::Config->new();
-    my $EncodeObject = Kernel::System::Encode->new(
-        ConfigObject => $ConfigObject,
-    );
-    my $LogObject = Kernel::System::Log->new(
-        ConfigObject => $ConfigObject,
-        EncodeObject => $EncodeObject,
-    );
     my $TransportObject = Kernel::GenericInterface::Transport->new(
-        ConfigObject       => $ConfigObject,
-        LogObject          => $LogObject,
-        DBObject           => $DBObject,
-        MainObject         => $MainObject,
-        TimeObject         => $TimeObject,
-        EncodeObject       => $EncodeObject,
 
         TransportConfig => {
             Type => 'HTTP::Test',
@@ -77,10 +60,7 @@ sub new {
     my $Self = {};
     bless( $Self, $Type );
 
-    for my $Needed (
-        qw(LogObject EncodeObject ConfigObject MainObject DebuggerObject TransportConfig)
-        )
-    {
+    for my $Needed (qw( DebuggerObject TransportConfig)) {
         $Self->{$Needed} = $Param{$Needed} || return {
             Success      => 0,
             ErrorMessage => "Got no $Needed!"
@@ -101,6 +81,7 @@ sub ProviderProcessRequest {
     my ( $Self, %Param ) = @_;
 
     if ( $Self->{TransportConfig}->{Config}->{Fail} ) {
+
         return {
             Success      => 0,
             ErrorMessage => "HTTP status code: 500",
@@ -108,7 +89,7 @@ sub ProviderProcessRequest {
         };
     }
 
-    my $ParamObject = Kernel::System::Web::Request->new( %{$Self} );
+    my $ParamObject = $Kernel::OM->Get('Kernel::System::Web::Request');
 
     my %Result;
     for my $ParamName ( $ParamObject->GetParamNames() ) {
@@ -121,6 +102,7 @@ sub ProviderProcessRequest {
     }
 
     if ( !%Result ) {
+
         return $Self->{DebuggerObject}->Error(
             Summary => 'No request data found.',
         );
@@ -146,6 +128,7 @@ sub ProviderGenerateResponse {
     my ( $Self, %Param ) = @_;
 
     if ( $Self->{TransportConfig}->{Config}->{Fail} ) {
+
         return {
             Success      => 0,
             ErrorMessage => 'Test response generation failed',
@@ -212,6 +195,7 @@ sub RequesterPerformRequest {
     my ( $Self, %Param ) = @_;
 
     if ( $Self->{TransportConfig}->{Config}->{Fail} ) {
+
         return {
             Success      => 0,
             ErrorMessage => "HTTP status code: 500",
@@ -286,10 +270,10 @@ sub request {    ## no critic
 
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<http://otrs.org/>).
+This software is part of the OTRS project (L<https://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (AGPL). If you
-did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
+the enclosed file COPYING for license information (GPL). If you
+did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
 
 =cut
