@@ -1,10 +1,9 @@
 # --
-# Kernel/Modules/AdminGenericInterfaceInvokerDefault.pm - provides a log view for admins
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Modules::AdminGenericInterfaceInvokerDefault;
@@ -13,6 +12,8 @@ use strict;
 use warnings;
 
 use Kernel::System::GenericInterface::Webservice;
+use Kernel::System::DynamicField;
+use Kernel::System::Event;
 
 use Kernel::System::VariableCheck qw(:all);
 
@@ -28,7 +29,12 @@ sub new {
         }
     }
 
-    $Self->{WebserviceObject} = Kernel::System::GenericInterface::Webservice->new( %{$Self} );
+    $Self->{WebserviceObject}   = Kernel::System::GenericInterface::Webservice->new( %{$Self} );
+    $Self->{DynamicFieldObject} = Kernel::System::DynamicField->new( %{$Self} );
+    $Self->{EventObject}        = Kernel::System::Event->new(
+        %{$Self},
+        DynamicFieldObject => $Self->{DynamicFieldObject},
+    );
 
     return $Self;
 }
@@ -568,8 +574,7 @@ sub _ShowScreen {
         );
     }
 
-    # get registered event triggers from the config
-    my $RegisteredEvents = $Self->{ConfigObject}->Get('GenericInterface::Invoker::Event');
+    my %RegisteredEvents = $Self->{EventObject}->EventList();
 
     my %InvokerEventLookup;
 
@@ -583,15 +588,15 @@ sub _ShowScreen {
         # set if is Synchonous or Asynchronous
         my $Asynchronous = 'No';
         if ( $Event->{Asynchronous} ) {
-            $Asynchronous = 'Yes'
+            $Asynchronous = 'Yes';
         }
 
         # set the event type ( event object like Article or Ticket) not currently in use
         # but left if is needed in future
         my $EventType;
         EVENTTYPE:
-        for my $Type ( sort keys %{$RegisteredEvents} ) {
-            if ( $RegisteredEvents->{$Type}->{ $Event->{Event} } ) {
+        for my $Type ( sort keys %RegisteredEvents ) {
+            if ( grep { $_ eq $Event->{Event} } @{ $RegisteredEvents{$Type} || [] } ) {
                 $EventType = $Type;
                 last EVENTTYPE;
             }
@@ -614,16 +619,14 @@ sub _ShowScreen {
 
     # create event trigger selectors (one for each type)
     TYPE:
-    for my $Type ( sort keys %{$RegisteredEvents} ) {
-        next TYPE if !$Type;
+    for my $Type ( sort keys %RegisteredEvents ) {
 
         # refresh event list for each event type
         my @EventList;
 
         EVENT:
-        for my $Event ( sort keys %{ $RegisteredEvents->{$Type} } ) {
-            next EVENT if !$RegisteredEvents->{$Type}->{$Event};
-            next EVENT if $InvokerEventLookup{$Event};
+        for my $Event ( @{ $RegisteredEvents{$Type} || [] } ) {
+            next EVENT if $InvokerEventLookup{$Event};    # only add events that are not used yet
             push @EventList, $Event;
         }
 
@@ -639,6 +642,7 @@ sub _ShowScreen {
             Name         => $Type . 'Event',
             Sort         => 'AlphanumericValue',
             PossibleNone => 0,
+            Title        => $Self->{LayoutObject}->{LanguageObject}->Translate('Event'),
             Class        => 'EventList GenericInterfaceSpacing ' . $EventListHidden,
         );
 
