@@ -1,12 +1,9 @@
 # --
-# Kernel/Modules/AdminGenericInterfaceWebservice.pm - provides a webservice view for admins
-# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
-# --
-# $Id: AdminGenericInterfaceWebservice.pm,v 1.34.2.1 2013-01-09 18:25:08 cr Exp $
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Modules::AdminGenericInterfaceWebservice;
@@ -14,15 +11,10 @@ package Kernel::Modules::AdminGenericInterfaceWebservice;
 use strict;
 use warnings;
 
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.34.2.1 $) [1];
-
 use Kernel::System::VariableCheck qw(:all);
 use Kernel::System::GenericInterface::Webservice;
 use Kernel::System::Valid;
-use YAML;
-
-use Kernel::System::VariableCheck qw(:all);
+use Kernel::System::YAML;
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -39,6 +31,7 @@ sub new {
     # create addtional objects
     $Self->{ValidObject}      = Kernel::System::Valid->new( %{$Self} );
     $Self->{WebserviceObject} = Kernel::System::GenericInterface::Webservice->new( %{$Self} );
+    $Self->{YAMLObject}       = Kernel::System::YAML->new( %{$Self} );
 
     # get configurations
     # get configured transports
@@ -119,7 +112,7 @@ sub Run {
         }
 
         # get parameter from web browser
-        my $GetParam = $Self->_GetParams;
+        my $GetParam = $Self->_GetParams();
 
         # set new confguration
         $WebserviceData->{Name}                                 = $GetParam->{Name};
@@ -202,12 +195,14 @@ sub Run {
         }
 
         # define notification
-        my $Notify = 'Webservice "%s" updated!", "' . $WebserviceData->{Name};
+        my $Notify = $Self->{LayoutObject}->{LanguageObject}->Translate(
+            'Webservice "%s" updated!',
+            $WebserviceData->{Name},
+        );
 
         # Save and finish button: go to Webservice.
         if ( $Self->{ParamObject}->GetParam( Param => 'ReturnToWebservice' ) ) {
-            my $RedirectURL
-                = "Action=AdminGenericInterfaceWebservice;";
+            my $RedirectURL = "Action=AdminGenericInterfaceWebservice;";
             return $Self->{LayoutObject}->Redirect(
                 OP => $RedirectURL,
             );
@@ -247,7 +242,7 @@ sub Run {
         my $WebserviceData;
 
         # get parameter from web browser
-        my $GetParam = $Self->_GetParams;
+        my $GetParam = $Self->_GetParams();
 
         # set new confguration
         $WebserviceData->{Name}                                 = $GetParam->{Name};
@@ -318,7 +313,10 @@ sub Run {
         $WebserviceID = $ID;
 
         # define notification
-        my $Notify = 'Webservice "%s" created!", "' . $WebserviceData->{Name};
+        my $Notify = $Self->{LayoutObject}->{LanguageObject}->Translate(
+            'Webservice "%s" created!',
+            $WebserviceData->{Name},
+        );
 
         # return to edit to continue changing the configuration
         return $Self->_ShowEdit(
@@ -356,7 +354,7 @@ sub Run {
         }
 
         # dump configuration into a YAML structure
-        my $YAMLContent = YAML::Dump( $WebserviceData->{Config} );
+        my $YAMLContent = $Self->{YAMLObject}->Dump( Data => $WebserviceData->{Config} );
 
         # return yaml to download
         my $YAMLFile = $WebserviceData->{Name};
@@ -462,7 +460,10 @@ sub Run {
         }
 
         # define notification
-        my $Notify = 'Webservice "%s" created!", "' . $WebserviceData->{Name};
+        my $Notify = $Self->{LayoutObject}->{LanguageObject}->Translate(
+            'Webservice "%s" created!',
+            $WebserviceData->{Name},
+        );
 
         # return to overview
         return $Self->_ShowOverview(
@@ -483,8 +484,7 @@ sub Run {
 
         # get the webservice config file from the http request
         my %ConfigFile = $Self->{ParamObject}->GetUploadAll(
-            Param  => 'ConfigFile',
-            Source => 'string',
+            Param => 'ConfigFile',
         );
 
         # check for file
@@ -497,20 +497,22 @@ sub Run {
         my $ImportedConfig;
 
         # read configuration from a YAML structure
-        # if there is an error in YAML it returns a had error eval is needed to handle the error
-        eval {
-            $ImportedConfig = YAML::Load( $ConfigFile{Content} );
-        };
+        $ImportedConfig = $Self->{YAMLObject}->Load( Data => $ConfigFile{Content} );
 
         # display any YAML error message as a normal otrs error message
-        if ($@) {
+        if ( !IsHashRefWithData($ImportedConfig) ) {
             return $Self->{LayoutObject}->ErrorScreen(
-                Message => $@,
+                Message => 'The imported file has not valid YAML content!'
+                    . ' Please check OTRS log for details',
             );
         }
 
         # check if imported configuration has current framework version otherwise update it
-        if ( $ImportedConfig->{FrameworkVersion} ne $Self->{FrameworkVersion} ) {
+        if (
+            !$ImportedConfig->{FrameworkVersion}
+            || $ImportedConfig->{FrameworkVersion} ne $Self->{FrameworkVersion}
+            )
+        {
             $ImportedConfig = $Self->_UpdateConfiguration( Configuration => $ImportedConfig );
         }
 
@@ -575,7 +577,10 @@ sub Run {
         );
 
         # define notification
-        my $Notify = 'Webservice "%s" created!", "' . $WebserviceData->{Name};
+        my $Notify = $Self->{LayoutObject}->{LanguageObject}->Translate(
+            'Webservice "%s" created!',
+            $WebserviceData->{Name},
+        );
 
         return $Self->_ShowOverview(
             %Param,
@@ -609,7 +614,10 @@ sub Run {
     if ($DeletedWebservice) {
 
         # define notification
-        $Notify = 'Webservice "%s" deleted!", "' . $DeletedWebservice;
+        $Notify = $Self->{LayoutObject}->{LanguageObject}->Translate(
+            'Webservice "%s" deleted!',
+            $DeletedWebservice,
+        );
 
     }
 
@@ -678,6 +686,23 @@ sub _ShowOverview {
                 ValidID => $Webservice->{ValidID},
             );
 
+            if ( !$Webservice->{Config} || !IsHashRefWithData( $Webservice->{Config} ) ) {
+
+                # write an error message to the OTRS log
+                $Self->{LogObject}->Log(
+                    Priority => 'error',
+                    Message  => "Configuration of WebserviceID $WebserviceID is invalid!",
+                );
+
+                # notify the user of problems loading this webservice
+                $Output .= $Self->{LayoutObject}->Notify(
+                    Priority => 'Error',
+                );
+
+                # continue loading the list of webservices
+                next WEBSERVICEID;
+            }
+
             # prepare data to output
             my $Data = {
                 ID           => $WebserviceID,
@@ -866,22 +891,25 @@ sub _ShowEdit {
 
     # set transports data
     my %GITransports;
-    for my $Transport ( keys %{ $Self->{GITransportConfig} } ) {
-        next if !$Transport;
+    TRANSPORT:
+    for my $Transport ( sort keys %{ $Self->{GITransportConfig} } ) {
+        next TRANSPORT if !$Transport;
         $GITransports{$Transport} = $Self->{GITransportConfig}->{$Transport}->{ConfigDialog};
     }
 
     # get operations data
     my %GIOperations;
-    for my $Operation ( keys %{ $Self->{GIOperationConfig} } ) {
-        next if !$Operation;
+    OPERATION:
+    for my $Operation ( sort keys %{ $Self->{GIOperationConfig} } ) {
+        next OPERATION if !$Operation;
         $GIOperations{$Operation} = $Self->{GIOperationConfig}->{$Operation}->{ConfigDialog};
     }
 
     # get operations data
     my %GIInvokers;
-    for my $Invoker ( keys %{ $Self->{GIInvokerConfig} } ) {
-        next if !$Invoker;
+    INVOKER:
+    for my $Invoker ( sort keys %{ $Self->{GIInvokerConfig} } ) {
+        next INVOKER if !$Invoker;
         $GIInvokers{$Invoker} = $Self->{GIInvokerConfig}->{$Invoker}->{ConfigDialog};
     }
 
@@ -916,7 +944,7 @@ sub _ShowEdit {
 
         my @TransportList;
 
-        for my $Transport ( keys %GITransports ) {
+        for my $Transport ( sort keys %GITransports ) {
             push @TransportList, $Transport;
         }
 
@@ -935,7 +963,7 @@ sub _ShowEdit {
 
         my @ControllerList;
 
-        for my $Action ( keys %GIControllers ) {
+        for my $Action ( sort keys %GIControllers ) {
             push @ControllerList, $Action;
         }
 
@@ -1002,8 +1030,7 @@ sub _ShowEdit {
             {
 
                 # get control information
-                my $ActionDetails
-                    = $CommTypeConfig{$CommunicationType}->{ActionsConfig}->{$ActionName};
+                my $ActionDetails = $CommTypeConfig{$CommunicationType}->{ActionsConfig}->{$ActionName};
 
                 # create output data
                 my %ActionData = (
@@ -1019,7 +1046,7 @@ sub _ShowEdit {
                 my $ControllerClass = '';
                 if ( !$GIControllers{ $ActionData{Controller} } ) {
                     $NoControllerFound = 1;
-                    $ControllerClass   = 'Error',
+                    $ControllerClass   = 'Error';
                 }
 
                 $Self->{LayoutObject}->Block(
