@@ -1,10 +1,9 @@
 # --
-# Kernel/Modules/AdminPGP.pm - to add/update/delete pgp keys
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Modules::AdminPGP;
@@ -166,8 +165,7 @@ sub Run {
             Value     => '',
         );
         my %UploadStuff = $Self->{ParamObject}->GetUploadAll(
-            Param  => 'FileUpload',
-            Source => 'String',
+            Param => 'FileUpload',
         );
         if ( !%UploadStuff ) {
             $Errors{FileUploadInvalid} = 'ServerError';
@@ -299,6 +297,19 @@ sub Run {
     # ------------------------------------------------------------ #
     else {
 
+        my $Output .= $Self->{LayoutObject}->Header();
+        $Output .= $Self->{LayoutObject}->NavigationBar();
+
+        if ( !$Self->{CryptObject} && $Self->{ConfigObject}->Get('PGP') ) {
+            $Output .= $Self->{LayoutObject}->Notify(
+                Priority => 'Error',
+                Data     => $Self->{LayoutObject}->{LanguageObject}->Translate( "Cannot create %s!", "CryptObject" ),
+                Link =>
+                    $Self->{LayoutObject}->{Baselink}
+                    . 'Action=AdminSysConfig;Subaction=Edit;SysConfigGroup=Framework;SysConfigSubGroup=Crypt::PGP',
+            );
+        }
+
         $Self->{LayoutObject}->Block( Name => 'Overview' );
         $Self->{LayoutObject}->Block( Name => 'ActionList' );
         $Self->{LayoutObject}->Block( Name => 'ActionSearch' );
@@ -324,12 +335,11 @@ sub Run {
                 Data => {},
             );
         }
-        my $Output .= $Self->{LayoutObject}->Header();
-        $Output .= $Self->{LayoutObject}->NavigationBar();
-        if ( $Self->{CryptObject}->Check() ) {
+
+        if ( $Self->{CryptObject} && $Self->{CryptObject}->Check() ) {
             $Output .= $Self->{LayoutObject}->Notify(
                 Priority => 'Error',
-                Data     => '$Text{"' . $Self->{CryptObject}->Check() . '"}',
+                Data     => $Self->{LayoutObject}->{LanguageObject}->Translate( $Self->{CryptObject}->Check() ),
             );
         }
         $Output .= $Self->{LayoutObject}->Output(
