@@ -1,10 +1,9 @@
 # --
-# Kernel/Modules/CustomerTicketOverview.pm - status for all open tickets
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Modules::CustomerTicketOverview;
@@ -64,6 +63,22 @@ sub new {
         FieldFilter => $Self->{DynamicFieldFilter} || {},
     );
 
+    # reduce the dynamic fields to only the ones that are desinged for customer interface
+    my @CustomerDynamicFields;
+    DYNAMICFIELD:
+    for my $DynamicFieldConfig ( @{ $Self->{DynamicField} } ) {
+        next DYNAMICFIELD if !IsHashRefWithData($DynamicFieldConfig);
+
+        my $IsCustomerInterfaceCapable = $Self->{BackendObject}->HasBehavior(
+            DynamicFieldConfig => $DynamicFieldConfig,
+            Behavior           => 'IsCustomerInterfaceCapable',
+        );
+        next DYNAMICFIELD if !$IsCustomerInterfaceCapable;
+
+        push @CustomerDynamicFields, $DynamicFieldConfig;
+    }
+    $Self->{DynamicField} = \@CustomerDynamicFields;
+
     # get params
     $Self->{Filter}  = $Self->{ParamObject}->GetParam( Param => 'Filter' )  || 'Open';
     $Self->{SortBy}  = $Self->{ParamObject}->GetParam( Param => 'SortBy' )  || 'Age';
@@ -95,12 +110,6 @@ sub Run {
         return $Output;
     }
 
-    # store last screen
-    $Self->{SessionObject}->UpdateSessionID(
-        SessionID => $Self->{SessionID},
-        Key       => 'LastScreenView',
-        Value     => $Self->{RequestedURL},
-    );
     $Self->{SessionObject}->UpdateSessionID(
         SessionID => $Self->{SessionID},
         Key       => 'LastScreenOverview',
@@ -114,35 +123,35 @@ sub Run {
                 Name   => 'All',
                 Prio   => 1000,
                 Search => {
-                    CustomerUserLogin => $Self->{UserID},
-                    OrderBy           => $Self->{OrderBy},
-                    SortBy            => $Self->{SortBy},
-                    CustomerUserID    => $Self->{UserID},
-                    Permission        => 'ro',
+                    CustomerUserLoginRaw => $Self->{UserID},
+                    OrderBy              => $Self->{OrderBy},
+                    SortBy               => $Self->{SortBy},
+                    CustomerUserID       => $Self->{UserID},
+                    Permission           => 'ro',
                 },
             },
             Open => {
                 Name   => 'Open',
                 Prio   => 1100,
                 Search => {
-                    CustomerUserLogin => $Self->{UserID},
-                    StateType         => 'Open',
-                    OrderBy           => $Self->{OrderBy},
-                    SortBy            => $Self->{SortBy},
-                    CustomerUserID    => $Self->{UserID},
-                    Permission        => 'ro',
+                    CustomerUserLoginRaw => $Self->{UserID},
+                    StateType            => 'Open',
+                    OrderBy              => $Self->{OrderBy},
+                    SortBy               => $Self->{SortBy},
+                    CustomerUserID       => $Self->{UserID},
+                    Permission           => 'ro',
                 },
             },
             Closed => {
                 Name   => 'Closed',
                 Prio   => 1200,
                 Search => {
-                    CustomerUserLogin => $Self->{UserID},
-                    StateType         => 'Closed',
-                    OrderBy           => $Self->{OrderBy},
-                    SortBy            => $Self->{SortBy},
-                    CustomerUserID    => $Self->{UserID},
-                    Permission        => 'ro',
+                    CustomerUserLoginRaw => $Self->{UserID},
+                    StateType            => 'Closed',
+                    OrderBy              => $Self->{OrderBy},
+                    SortBy               => $Self->{SortBy},
+                    CustomerUserID       => $Self->{UserID},
+                    Permission           => 'ro',
                 },
             },
         },
@@ -155,7 +164,7 @@ sub Run {
                 Name   => 'All',
                 Prio   => 1000,
                 Search => {
-                    CustomerID =>
+                    CustomerIDRaw =>
                         [ $Self->{UserObject}->CustomerIDs( User => $Self->{UserLogin} ) ],
                     OrderBy        => $Self->{OrderBy},
                     SortBy         => $Self->{SortBy},
@@ -167,7 +176,7 @@ sub Run {
                 Name   => 'Open',
                 Prio   => 1100,
                 Search => {
-                    CustomerID =>
+                    CustomerIDRaw =>
                         [ $Self->{UserObject}->CustomerIDs( User => $Self->{UserLogin} ) ],
                     StateType      => 'Open',
                     OrderBy        => $Self->{OrderBy},
@@ -180,7 +189,7 @@ sub Run {
                 Name   => 'Closed',
                 Prio   => 1200,
                 Search => {
-                    CustomerID =>
+                    CustomerIDRaw =>
                         [ $Self->{UserObject}->CustomerIDs( User => $Self->{UserLogin} ) ],
                     StateType      => 'Closed',
                     OrderBy        => $Self->{OrderBy},
@@ -219,16 +228,6 @@ sub Run {
     for my $Filter ( sort keys %{ $Filters{ $Self->{Subaction} } } ) {
         $Counter++;
 
-        # quote all CustomerIDs
-        my $CustomerIDs = $Filters{ $Self->{Subaction} }->{$Filter}->{Search}->{CustomerID};
-        if ( IsArrayRefWithData($CustomerIDs) ) {
-            for my $CustomerID ( @{$CustomerIDs} ) {
-                $CustomerID = $Self->{DBObject}->QueryStringEscape(
-                    QueryString => $CustomerID,
-                );
-            }
-        }
-
         my $Count = $Self->{TicketObject}->TicketSearch(
             %{ $Filters{ $Self->{Subaction} }->{$Filter}->{Search} },
             %SearchInArchive,
@@ -330,8 +329,9 @@ sub Run {
         my $TitleSort  = '';
         my $AgeSort    = '';
         my $QueueSort  = '';
+        my $OwnerSort  = '';
 
-        # this sets the opposit to the $OrderBy
+        # this sets the opposite to the $OrderBy
         if ( $OrderBy eq 'Down' ) {
             $Sort = 'SortAscending';
         }
@@ -354,6 +354,9 @@ sub Run {
         elsif ( $Self->{SortBy} eq 'Queue' ) {
             $QueueSort = $Sort;
         }
+        elsif ( $Self->{SortBy} eq 'Owner' ) {
+            $OwnerSort = $Sort;
+        }
         $Self->{LayoutObject}->Block(
             Name => 'Filled',
             Data => {
@@ -371,6 +374,11 @@ sub Run {
         if ( $Self->{Owner} ) {
             $Self->{LayoutObject}->Block(
                 Name => 'OverviewNavBarPageOwner',
+                Data => {
+                    OrderBy   => $OrderBy,
+                    OwnerSort => $OwnerSort,
+                    Filter    => $Self->{Filter},
+                },
             );
         }
 
@@ -425,8 +433,9 @@ sub Run {
             my $Label = $DynamicFieldConfig->{Label};
 
             # get field sortable condition
-            my $IsSortable = $Self->{BackendObject}->IsSortable(
+            my $IsSortable = $Self->{BackendObject}->HasBehavior(
                 DynamicFieldConfig => $DynamicFieldConfig,
+                Behavior           => 'IsSortable',
             );
 
             if ($IsSortable) {
@@ -528,7 +537,6 @@ sub Run {
             %{ $Filters{ $Self->{Subaction} }->{ $Self->{Filter} }->{Search} },
             %SearchInArchive,
             Result => 'ARRAY',
-            Limit  => 1_000,
         );
 
         # show tickets
@@ -589,8 +597,8 @@ sub ShowTicketStatus {
         for my $ArticleID ( reverse @ArticleIDs ) {
             my %CurrentArticle = $Self->{TicketObject}->ArticleGet( ArticleID => $ArticleID );
 
-            # check for non-internal article
-            next ARTICLEID if $CurrentArticle{ArticleType} =~ m{internal}smx;
+            # check for non-internal and non-chat article
+            next ARTICLEID if $CurrentArticle{ArticleType} =~ m{internal|chat}smx;
 
             # check for customer article
             if ( $CurrentArticle{SenderType} eq 'customer' ) {
@@ -638,7 +646,7 @@ sub ShowTicketStatus {
             Age   => $Ticket{Age},
             Space => ' '
         ) || 0;
-        $Article{Body} = $Self->{LayoutObject}->{LanguageObject}->Get('This item has no articles yet.');
+        $Article{Body} = $Self->{LayoutObject}->{LanguageObject}->Translate('This item has no articles yet.');
     }
 
     # otherwise return article information
