@@ -1,11 +1,9 @@
 # --
-# Kernel/Output/HTML/DashboardIFrame.pm
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
-# --
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Output::HTML::DashboardIFrame;
@@ -13,6 +11,11 @@ package Kernel::Output::HTML::DashboardIFrame;
 use strict;
 use warnings;
 
+# prevent 'Used once' warning
+use Kernel::System::ObjectManager;
+
+our $ObjectManagerDisabled = 1;
+
 sub new {
     my ( $Type, %Param ) = @_;
 
@@ -20,12 +23,9 @@ sub new {
     my $Self = {%Param};
     bless( $Self, $Type );
 
-    # get needed objects
-    for (
-        qw(Config Name ConfigObject LogObject DBObject LayoutObject ParamObject TicketObject UserID)
-        )
-    {
-        die "Got no $_!" if ( !$Self->{$_} );
+    # get needed parameters
+    for my $Needed (qw(Config Name UserID)) {
+        die "Got no $Needed!" if ( !$Self->{$Needed} );
     }
 
     return $Self;
@@ -52,7 +52,7 @@ sub Run {
     my $Title = $Self->{Config}->{Title} || '';
     $Title =~ s/\s/_/smx;
 
-    my $Content = $Self->{LayoutObject}->Output(
+    my $Content = $Kernel::OM->Get('Kernel::Output::HTML::Layout')->Output(
         TemplateFile => 'AgentDashboardIFrame',
         Data         => {
             %{ $Self->{Config} },
