@@ -1,10 +1,9 @@
 # --
-# Kernel/Output/HTML/ToolBarTicketResponsible.pm
-# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Output::HTML::ToolBarTicketResponsible;
@@ -76,6 +75,10 @@ sub Run {
     my $ClassNew     = $Param{Config}->{CssClassNew};
     my $ClassReached = $Param{Config}->{CssClassReached};
 
+    my $Icon        = $Param{Config}->{Icon};
+    my $IconNew     = $Param{Config}->{IconNew};
+    my $IconReached = $Param{Config}->{IconReached};
+
     my $URL = $Self->{LayoutObject}->{Baselink};
     my %Return;
     my $Priority = $Param{Config}->{Priority};
@@ -85,8 +88,9 @@ sub Run {
             Description => 'Responsible Tickets New',
             Count       => $CountNew,
             Class       => $ClassNew,
+            Icon        => $IconNew,
             Link        => $URL . 'Action=AgentTicketResponsibleView;Filter=New',
-            AccessKey   => 'r',
+            AccessKey   => $Param{Config}->{AccessKeyNew} || '',
         };
     }
     if ($CountReached) {
@@ -95,8 +99,9 @@ sub Run {
             Description => 'Responsible Tickets Reminder Reached',
             Count       => $CountReached,
             Class       => $ClassReached,
+            Icon        => $IconReached,
             Link        => $URL . 'Action=AgentTicketResponsibleView;Filter=ReminderReached',
-            AccessKey   => 'r',
+            AccessKey   => $Param{Config}->{AccessKeyReached} || '',
         };
     }
     if ($Count) {
@@ -105,8 +110,9 @@ sub Run {
             Description => 'Responsible Tickets Total',
             Count       => $Count,
             Class       => $Class,
+            Icon        => $Icon,
             Link        => $URL . 'Action=AgentTicketResponsibleView',
-            AccessKey   => 'r',
+            AccessKey   => $Param{Config}->{AccessKey} || '',
         };
     }
     return %Return;
