@@ -1,12 +1,9 @@
 # --
-# Kernel/System/PostMaster/NewTicket.pm - sub part of PostMaster.pm
-# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
-# --
-# $Id: NewTicket.pm,v 1.86 2012-04-19 21:12:39 mb Exp $
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::System::PostMaster::NewTicket;
@@ -14,11 +11,20 @@ package Kernel::System::PostMaster::NewTicket;
 use strict;
 use warnings;
 
-use Kernel::System::AutoResponse;
-use Kernel::System::CustomerUser;
-
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.86 $) [1];
+our @ObjectDependencies = (
+    'Kernel::Config',
+    'Kernel::System::CustomerUser',
+    'Kernel::System::DynamicField',
+    'Kernel::System::DynamicField::Backend',
+    'Kernel::System::LinkObject',
+    'Kernel::System::Log',
+    'Kernel::System::Priority',
+    'Kernel::System::Queue',
+    'Kernel::System::State',
+    'Kernel::System::Ticket',
+    'Kernel::System::Time',
+    'Kernel::System::User',
+);
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -27,18 +33,11 @@ sub new {
     my $Self = {};
     bless( $Self, $Type );
 
+    # get parser object
+    $Self->{ParserObject} = $Param{ParserObject} || die "Got no ParserObject!";
+
     $Self->{Debug} = $Param{Debug} || 0;
 
-    # get all objects
-    for (
-        qw(DBObject ConfigObject TicketObject LogObject ParserObject TimeObject QueueObject StateObject PriorityObject)
-        )
-    {
-        $Self->{$_} = $Param{$_} || die 'Got no $_';
-    }
-
-    $Self->{CustomerUserObject} = Kernel::System::CustomerUser->new(%Param);
-
     return $Self;
 }
 
@@ -46,9 +45,12 @@ sub Run {
     my ( $Self, %Param ) = @_;
 
     # check needed stuff
-    for (qw(InmailUserID GetParam)) {
-        if ( !$Param{$_} ) {
-            $Self->{LogObject}->Log( Priority => 'error', Message => "Need $_!" );
+    for my $Needed (qw(InmailUserID GetParam)) {
+        if ( !$Param{$Needed} ) {
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
+                Priority => 'error',
+                Message  => "Need $Needed!"
+            );
             return;
         }
     }
@@ -58,33 +60,46 @@ sub Run {
 
     # get queue id and name
     my $QueueID = $Param{QueueID} || die "need QueueID!";
-    my $Queue = $Self->{QueueObject}->QueueLookup( QueueID => $QueueID );
+    my $Queue = $Kernel::OM->Get('Kernel::System::Queue')->QueueLookup(
+        QueueID => $QueueID,
+    );
+
+    # get config object
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
 
     # get state
-    my $State = $Self->{ConfigObject}->Get('PostmasterDefaultState') || 'new';
+    my $State = $ConfigObject->Get('PostmasterDefaultState') || 'new';
     if ( $GetParam{'X-OTRS-State'} ) {
-        my $StateID = $Self->{StateObject}->StateLookup( State => $GetParam{'X-OTRS-State'} );
+
+        my $StateID = $Kernel::OM->Get('Kernel::System::State')->StateLookup(
+            State => $GetParam{'X-OTRS-State'},
+        );
+
         if ($StateID) {
             $State = $GetParam{'X-OTRS-State'};
         }
         else {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
-                Message => "State $GetParam{'X-OTRS-State'} does not exist, falling back to $State!"
+                Message  => "State $GetParam{'X-OTRS-State'} does not exist, falling back to $State!"
             );
         }
     }
 
     # get priority
-    my $Priority = $Self->{ConfigObject}->Get('PostmasterDefaultPriority') || '3 normal';
+    my $Priority = $ConfigObject->Get('PostmasterDefaultPriority') || '3 normal';
+
     if ( $GetParam{'X-OTRS-Priority'} ) {
-        my $PriorityID
-            = $Self->{PriorityObject}->PriorityLookup( Priority => $GetParam{'X-OTRS-Priority'} );
+
+        my $PriorityID = $Kernel::OM->Get('Kernel::System::Priority')->PriorityLookup(
+            Priority => $GetParam{'X-OTRS-Priority'},
+        );
+
         if ($PriorityID) {
             $Priority = $GetParam{'X-OTRS-Priority'};
         }
         else {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message =>
                     "Priority $GetParam{'X-OTRS-Priority'} does not exist, falling back to $Priority!"
@@ -93,18 +108,26 @@ sub Run {
     }
 
     # get sender email
-    my @EmailAddresses = $Self->{ParserObject}->SplitAddressLine( Line => $GetParam{From}, );
-    for (@EmailAddresses) {
-        $GetParam{SenderEmailAddress} = $Self->{ParserObject}->GetEmailAddress( Email => $_, );
+    my @EmailAddresses = $Self->{ParserObject}->SplitAddressLine(
+        Line => $GetParam{From},
+    );
+    for my $Address (@EmailAddresses) {
+        $GetParam{SenderEmailAddress} = $Self->{ParserObject}->GetEmailAddress(
+            Email => $Address,
+        );
     }
 
     # get customer id (sender email) if there is no customer id given
     if ( !$GetParam{'X-OTRS-CustomerNo'} && $GetParam{'X-OTRS-CustomerUser'} ) {
 
+        # get customer user object
+        my $CustomerUserObject = $Kernel::OM->Get('Kernel::System::CustomerUser');
+
         # get customer user data form X-OTRS-CustomerUser
-        my %CustomerData = $Self->{CustomerUserObject}->CustomerUserDataGet(
+        my %CustomerData = $CustomerUserObject->CustomerUserDataGet(
             User => $GetParam{'X-OTRS-CustomerUser'},
         );
+
         if (%CustomerData) {
             $GetParam{'X-OTRS-CustomerNo'} = $CustomerData{UserCustomerID};
         }
@@ -112,23 +135,34 @@ sub Run {
 
     # get customer user data form From: (sender address)
     if ( !$GetParam{'X-OTRS-CustomerUser'} ) {
+
         my %CustomerData;
         if ( $GetParam{From} ) {
+
             my @EmailAddresses = $Self->{ParserObject}->SplitAddressLine(
                 Line => $GetParam{From},
             );
-            for (@EmailAddresses) {
-                $GetParam{EmailForm} = $Self->{ParserObject}->GetEmailAddress(
-                    Email => $_,
+
+            for my $Address (@EmailAddresses) {
+                $GetParam{EmailFrom} = $Self->{ParserObject}->GetEmailAddress(
+                    Email => $Address,
                 );
             }
-            my %List = $Self->{CustomerUserObject}->CustomerSearch(
-                PostMasterSearch => lc( $GetParam{EmailForm} ),
-            );
-            for ( keys %List ) {
-                %CustomerData = $Self->{CustomerUserObject}->CustomerUserDataGet(
-                    User => $_,
+
+            if ( $GetParam{EmailFrom} ) {
+
+                # get customer user object
+                my $CustomerUserObject = $Kernel::OM->Get('Kernel::System::CustomerUser');
+
+                my %List = $CustomerUserObject->CustomerSearch(
+                    PostMasterSearch => lc( $GetParam{EmailFrom} ),
                 );
+
+                for my $UserLogin ( sort keys %List ) {
+                    %CustomerData = $CustomerUserObject->CustomerUserDataGet(
+                        User => $UserLogin,
+                    );
+                }
             }
         }
 
@@ -137,20 +171,20 @@ sub Run {
             $GetParam{'X-OTRS-CustomerUser'} = $CustomerData{UserLogin};
 
             # notice that UserLogin is from customer source backend
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'notice',
                 Message  => "Take UserLogin ($CustomerData{UserLogin}) from "
-                    . "customer source backend based on ($GetParam{'EmailForm'}).",
+                    . "customer source backend based on ($GetParam{'EmailFrom'}).",
             );
         }
         if ( $CustomerData{UserCustomerID} && !$GetParam{'X-OTRS-CustomerNo'} ) {
             $GetParam{'X-OTRS-CustomerNo'} = $CustomerData{UserCustomerID};
 
             # notice that UserCustomerID is from customer source backend
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'notice',
                 Message  => "Take UserCustomerID ($CustomerData{UserCustomerID})"
-                    . " from customer source backend based on ($GetParam{'EmailForm'}).",
+                    . " from customer source backend based on ($GetParam{'EmailFrom'}).",
             );
         }
     }
@@ -165,9 +199,37 @@ sub Run {
         $GetParam{'X-OTRS-CustomerUser'} = $GetParam{SenderEmailAddress};
     }
 
+    # get ticket owner
+    my $OwnerID = $GetParam{'X-OTRS-OwnerID'} || $Param{InmailUserID};
+    if ( $GetParam{'X-OTRS-Owner'} ) {
+
+        my $TmpOwnerID = $Kernel::OM->Get('Kernel::System::User')->UserLookup(
+            UserLogin => $GetParam{'X-OTRS-Owner'},
+        );
+
+        $OwnerID = $TmpOwnerID || $OwnerID;
+    }
+
+    my %Opts;
+    if ( $GetParam{'X-OTRS-ResponsibleID'} ) {
+        $Opts{ResponsibleID} = $GetParam{'X-OTRS-ResponsibleID'};
+    }
+
+    if ( $GetParam{'X-OTRS-Responsible'} ) {
+
+        my $TmpResponsibleID = $Kernel::OM->Get('Kernel::System::User')->UserLookup(
+            UserLogin => $GetParam{'X-OTRS-Responsible'},
+        );
+
+        $Opts{ResponsibleID} = $TmpResponsibleID || $Opts{ResponsibleID};
+    }
+
+    # get ticket object
+    my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
+
     # create new ticket
-    my $NewTn    = $Self->{TicketObject}->TicketCreateNumber();
-    my $TicketID = $Self->{TicketObject}->TicketCreate(
+    my $NewTn    = $TicketObject->TicketCreateNumber();
+    my $TicketID = $TicketObject->TicketCreate(
         TN           => $NewTn,
         Title        => $GetParam{Subject},
         QueueID      => $QueueID,
@@ -179,8 +241,9 @@ sub Run {
         SLA          => $GetParam{'X-OTRS-SLA'} || '',
         CustomerID   => $GetParam{'X-OTRS-CustomerNo'},
         CustomerUser => $GetParam{'X-OTRS-CustomerUser'},
-        OwnerID      => $Param{InmailUserID},
+        OwnerID      => $OwnerID,
         UserID       => $Param{InmailUserID},
+        %Opts,
     );
 
     if ( !$TicketID ) {
@@ -196,18 +259,51 @@ sub Run {
         print "State: $State\n";
         print "CustomerID: $GetParam{'X-OTRS-CustomerNo'}\n";
         print "CustomerUser: $GetParam{'X-OTRS-CustomerUser'}\n";
-        for (qw(Type Service SLA Lock)) {
+        for my $Value (qw(Type Service SLA Lock)) {
 
-            if ( $GetParam{ 'X-OTRS-' . $_ } ) {
-                print "Type: " . $GetParam{ 'X-OTRS-' . $_ } . "\n";
+            if ( $GetParam{ 'X-OTRS-' . $Value } ) {
+                print "Type: " . $GetParam{ 'X-OTRS-' . $Value } . "\n";
             }
         }
     }
 
     # set pending time
     if ( $GetParam{'X-OTRS-State-PendingTime'} ) {
-        my $Set = $Self->{TicketObject}->TicketPendingTimeSet(
-            String   => $GetParam{'X-OTRS-State-PendingTime'},
+
+  # You can specify absolute dates like "2010-11-20 00:00:00" or relative dates, based on the arrival time of the email.
+  # Use the form "+ $Number $Unit", where $Unit can be 's' (seconds), 'm' (minutes), 'h' (hours) or 'd' (days).
+  # Only one unit can be specified. Examples of valid settings: "+50s" (pending in 50 seconds), "+30m" (30 minutes),
+  # "+12d" (12 days). Note that settings like "+1d 12h" are not possible. You can specify "+36h" instead.
+
+        my $TargetTimeStamp = $GetParam{'X-OTRS-State-PendingTime'};
+
+        my ( $Sign, $Number, $Unit ) = $TargetTimeStamp =~ m{^\s*([+-]?)\s*(\d+)\s*([smhd]?)\s*$}smx;
+
+        if ($Number) {
+            $Sign ||= '+';
+            $Unit ||= 's';
+
+            my $Seconds = $Sign eq '-' ? ( $Number * -1 ) : $Number;
+
+            my %UnitMultiplier = (
+                s => 1,
+                m => 60,
+                h => 60 * 60,
+                d => 60 * 60 * 24,
+            );
+
+            $Seconds = $Seconds * $UnitMultiplier{$Unit};
+
+            # get time object
+            my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+
+            $TargetTimeStamp = $TimeObject->SystemTime2TimeStamp(
+                SystemTime => $TimeObject->SystemTime() + $Seconds,
+            );
+        }
+
+        my $Set = $TicketObject->TicketPendingTimeSet(
+            String   => $TargetTimeStamp,
             TicketID => $TicketID,
             UserID   => $Param{InmailUserID},
         );
@@ -218,12 +314,16 @@ sub Run {
         }
     }
 
+    # get dynamic field objects
+    my $DynamicFieldObject        = $Kernel::OM->Get('Kernel::System::DynamicField');
+    my $DynamicFieldBackendObject = $Kernel::OM->Get('Kernel::System::DynamicField::Backend');
+
     # dynamic fields
     my $DynamicFieldList =
-        $Self->{TicketObject}->{DynamicFieldObject}->DynamicFieldList(
+        $DynamicFieldObject->DynamicFieldList(
         Valid      => 1,
         ResultType => 'HASH',
-        ObjectType => 'Ticket'
+        ObjectType => 'Ticket',
         );
 
     # set dynamic fields for Ticket object type
@@ -236,12 +336,11 @@ sub Run {
         if ( $GetParam{$Key} ) {
 
             # get dynamic field config
-            my $DynamicFieldGet
-                = $Self->{TicketObject}->{DynamicFieldObject}->DynamicFieldGet(
+            my $DynamicFieldGet = $DynamicFieldObject->DynamicFieldGet(
                 ID => $DynamicFieldID,
-                );
+            );
 
-            $Self->{TicketObject}->{DynamicFieldBackendObject}->ValueSet(
+            $DynamicFieldBackendObject->ValueSet(
                 DynamicFieldConfig => $DynamicFieldGet,
                 ObjectID           => $TicketID,
                 Value              => $GetParam{$Key},
@@ -258,6 +357,7 @@ sub Run {
     my %DynamicFieldListReversed = reverse %{$DynamicFieldList};
 
     # set ticket free text
+    # for backward compatibility (should be removed in a future version)
     my %Values =
         (
         'X-OTRS-TicketKey'   => 'TicketFreeKey',
@@ -269,11 +369,11 @@ sub Run {
             if ( $GetParam{$Key} && $DynamicFieldListReversed{ $Values{$Item} . $Count } ) {
 
                 # get dynamic field config
-                my $DynamicFieldGet = $Self->{TicketObject}->{DynamicFieldObject}->DynamicFieldGet(
+                my $DynamicFieldGet = $DynamicFieldObject->DynamicFieldGet(
                     ID => $DynamicFieldListReversed{ $Values{$Item} . $Count },
                 );
                 if ($DynamicFieldGet) {
-                    my $Success = $Self->{TicketObject}->{DynamicFieldBackendObject}->ValueSet(
+                    my $Success = $DynamicFieldBackendObject->ValueSet(
                         DynamicFieldConfig => $DynamicFieldGet,
                         ObjectID           => $TicketID,
                         Value              => $GetParam{$Key},
@@ -289,20 +389,29 @@ sub Run {
     }
 
     # set ticket free time
+    # for backward compatibility (should be removed in a future version)
     for my $Count ( 1 .. 6 ) {
+
         my $Key = 'X-OTRS-TicketTime' . $Count;
+
         if ( $GetParam{$Key} ) {
-            my $SystemTime = $Self->{TimeObject}->TimeStamp2SystemTime(
+
+            # get time object
+            my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+
+            my $SystemTime = $TimeObject->TimeStamp2SystemTime(
                 String => $GetParam{$Key},
             );
+
             if ( $SystemTime && $DynamicFieldListReversed{ 'TicketFreeTime' . $Count } ) {
 
                 # get dynamic field config
-                my $DynamicFieldGet = $Self->{TicketObject}->{DynamicFieldObject}->DynamicFieldGet(
+                my $DynamicFieldGet = $DynamicFieldObject->DynamicFieldGet(
                     ID => $DynamicFieldListReversed{ 'TicketFreeTime' . $Count },
                 );
+
                 if ($DynamicFieldGet) {
-                    my $Success = $Self->{TicketObject}->{DynamicFieldBackendObject}->ValueSet(
+                    my $Success = $DynamicFieldBackendObject->ValueSet(
                         DynamicFieldConfig => $DynamicFieldGet,
                         ObjectID           => $TicketID,
                         Value              => $GetParam{$Key},
@@ -318,7 +427,7 @@ sub Run {
     }
 
     # do article db insert
-    my $ArticleID = $Self->{TicketObject}->ArticleCreate(
+    my $ArticleID = $TicketObject->ArticleCreate(
         TicketID         => $TicketID,
         ArticleType      => $GetParam{'X-OTRS-ArticleType'},
         SenderType       => $GetParam{'X-OTRS-SenderType'},
@@ -342,11 +451,11 @@ sub Run {
 
     # close ticket if article create failed!
     if ( !$ArticleID ) {
-        $Self->{TicketObject}->TicketDelete(
+        $TicketObject->TicketDelete(
             TicketID => $TicketID,
             UserID   => $Param{InmailUserID},
         );
-        $Self->{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => "Can't process email with MessageID <$GetParam{'Message-ID'}>! "
                 . "Please create a bug report with this email (From: $GetParam{From}, Located "
@@ -355,25 +464,37 @@ sub Run {
         return;
     }
 
+    if ( $Param{LinkToTicketID} ) {
+
+        my $SourceKey = $Param{LinkToTicketID};
+        my $TargetKey = $TicketID;
+
+        $Kernel::OM->Get('Kernel::System::LinkObject')->LinkAdd(
+            SourceObject => 'Ticket',
+            SourceKey    => $SourceKey,
+            TargetObject => 'Ticket',
+            TargetKey    => $TargetKey,
+            Type         => 'Normal',
+            State        => 'Valid',
+            UserID       => $Param{InmailUserID},
+        );
+    }
+
     # debug
     if ( $Self->{Debug} > 0 ) {
-        print "From: $GetParam{From}\n";
-        print "ReplyTo: $GetParam{ReplyTo}\n" if ( $GetParam{ReplyTo} );
-        print "To: $GetParam{To}\n";
-        print "Cc: $GetParam{Cc}\n" if ( $GetParam{Cc} );
-        print "Subject: $GetParam{Subject}\n";
-        print "MessageID: $GetParam{'Message-ID'}\n";
-        print "Queue: $Queue\n";
-        print "SenderType: $GetParam{'X-OTRS-SenderType'}\n";
-        print "ArticleType: $GetParam{'X-OTRS-ArticleType'}\n";
+        ATTRIBUTE:
+        for my $Attribute ( sort keys %GetParam ) {
+            next ATTRIBUTE if !$GetParam{$Attribute};
+            print "$Attribute: $GetParam{$Attribute}\n";
+        }
     }
 
     # dynamic fields
     $DynamicFieldList =
-        $Self->{TicketObject}->{DynamicFieldObject}->DynamicFieldList(
+        $DynamicFieldObject->DynamicFieldList(
         Valid      => 1,
         ResultType => 'HASH',
-        ObjectType => 'Article'
+        ObjectType => 'Article',
         );
 
     # set dynamic fields for Article object type
@@ -385,12 +506,11 @@ sub Run {
         if ( $GetParam{$Key} ) {
 
             # get dynamic field config
-            my $DynamicFieldGet
-                = $Self->{TicketObject}->{DynamicFieldObject}->DynamicFieldGet(
+            my $DynamicFieldGet = $DynamicFieldObject->DynamicFieldGet(
                 ID => $DynamicFieldID,
-                );
+            );
 
-            $Self->{TicketObject}->{DynamicFieldBackendObject}->ValueSet(
+            $DynamicFieldBackendObject->ValueSet(
                 DynamicFieldConfig => $DynamicFieldGet,
                 ObjectID           => $ArticleID,
                 Value              => $GetParam{$Key},
@@ -407,6 +527,7 @@ sub Run {
     %DynamicFieldListReversed = reverse %{$DynamicFieldList};
 
     # set free article text
+    # for backward compatibility (should be removed in a future version)
     %Values =
         (
         'X-OTRS-ArticleKey'   => 'ArticleFreeKey',
@@ -418,11 +539,11 @@ sub Run {
             if ( $GetParam{$Key} && $DynamicFieldListReversed{ $Values{$Item} . $Count } ) {
 
                 # get dynamic field config
-                my $DynamicFieldGet = $Self->{TicketObject}->{DynamicFieldObject}->DynamicFieldGet(
+                my $DynamicFieldGet = $DynamicFieldObject->DynamicFieldGet(
                     ID => $DynamicFieldListReversed{ $Values{$Item} . $Count },
                 );
                 if ($DynamicFieldGet) {
-                    my $Success = $Self->{TicketObject}->{DynamicFieldBackendObject}->ValueSet(
+                    my $Success = $DynamicFieldBackendObject->ValueSet(
                         DynamicFieldConfig => $DynamicFieldGet,
                         ObjectID           => $ArticleID,
                         Value              => $GetParam{$Key},
@@ -438,7 +559,7 @@ sub Run {
     }
 
     # write plain email to the storage
-    $Self->{TicketObject}->ArticleWritePlain(
+    $TicketObject->ArticleWritePlain(
         ArticleID => $ArticleID,
         Email     => $Self->{ParserObject}->GetPlainEmail(),
         UserID    => $Param{InmailUserID},
@@ -446,12 +567,13 @@ sub Run {
 
     # write attachments to the storage
     for my $Attachment ( $Self->{ParserObject}->GetAttachments() ) {
-        $Self->{TicketObject}->ArticleWriteAttachment(
+        $TicketObject->ArticleWriteAttachment(
             Filename           => $Attachment->{Filename},
             Content            => $Attachment->{Content},
             ContentType        => $Attachment->{ContentType},
             ContentID          => $Attachment->{ContentID},
             ContentAlternative => $Attachment->{ContentAlternative},
+            Disposition        => $Attachment->{Disposition},
             ArticleID          => $ArticleID,
             UserID             => $Param{InmailUserID},
         );
