@@ -1,24 +1,19 @@
-#!/usr/bin/perl -w
+#!/usr/bin/perl
 # --
-# otrs.Scheduler4win.pl - provides Scheduler Daemon control for Microsoft Windows OS
-# Copyright (C) 2001-2014 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
 # --
-# $Id: otrs.Scheduler4win.pl,v 1.22 2011-07-18 19:46:20 cr Exp $
-# --
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU AFFERO General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU Affero General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
-# or see http://www.gnu.org/licenses/agpl.txt.
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 use strict;
@@ -30,19 +25,22 @@ use warnings;
 use File::Basename;
 use FindBin qw($RealBin);
 use lib dirname($RealBin);
-
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.22 $) [1];
+use lib dirname($RealBin) . '/Kernel/cpan-lib';
+use lib dirname($RealBin) . '/Custom';
 
 use Getopt::Std;
-use Kernel::Config;
-use Kernel::System::Encode;
-use Kernel::System::Log;
-use Kernel::System::Main;
-use Kernel::System::Time;
-use Kernel::System::DB;
-use Kernel::System::PID;
-use Kernel::Scheduler;
+
+use Kernel::System::ObjectManager;
+use Kernel::System::VariableCheck qw(:all);
+
+local $Kernel::OM = Kernel::System::ObjectManager->new(
+    'Kernel::System::Log' => {
+        LogPrefix => 'OTRS-otrs.Scheduler',
+    },
+);
+
+# defie PID name
+my $PIDName = 'otrs.Scheduler';
 
 # to store service name
 my $Service = 'OTRSScheduler';
@@ -51,8 +49,8 @@ my $Service = 'OTRSScheduler';
 my $ServiceStatus = {};
 
 # get options
-my %Opts = ();
-getopt( 'haf', \%Opts );
+my %Opts;
+getopt( 'aw', \%Opts );
 
 BEGIN {
 
@@ -68,8 +66,12 @@ use Win32::Daemon;
 use Win32::Service;
 
 # starting and stopping can only be done with UAC enabled
-if ( $Opts{a} && ( $Opts{a} eq "start" || $Opts{a} eq "stop" ) ) {
-    require Win32;
+if (
+    ( $Opts{a} && ( $Opts{a} eq "start" || $Opts{a} eq "stop" ) )
+    || $Opts{w}
+    )
+{
+    require Win32;    ## no critic
 
     if ( !Win32::IsAdminUser() ) {
         print "To be able to start or stop the Scheduler, call the script with UAC enabled.\n";
@@ -80,83 +82,79 @@ if ( $Opts{a} && ( $Opts{a} eq "start" || $Opts{a} eq "stop" ) ) {
 
 # help option
 if ( $Opts{h} ) {
-    _help();
-    exit 0;
+    _Help();
+    exit 1;
+}
+
+# check if watch dog mode is requested
+if ( $Opts{w} ) {
+
+    my $ExitCode = _WatchDog();
+    exit $ExitCode;
 }
 
 # check if a stop request is sent
 if ( $Opts{a} && $Opts{a} eq "stop" ) {
 
-    # create common objects
-    my %CommonObject = _CommonObjects();
+    my $Force = $Opts{f} ? 1 : '';
 
-    if ( $Opts{f} ) {
-
-        # delete process ID lock
-        my $PIDDelSuccess = $CommonObject{PIDObject}->PIDDelete( Name => 'otrs.Scheduler' );
-    }
-    else {
-
-        # get the process ID
-        my %PID = $CommonObject{PIDObject}->PIDGet(
-            Name => 'otrs.Scheduler',
-        );
-
-        # no process ID means that is not running
-        if ( !%PID ) {
-            print "OTRS scheduler was already in stopped state.\n";
-            exit 1;
-        }
-    }
-
-    # stop the scheduler service (same as "stop"" in service control manger)
-    # cant use Win32::Daemon because is called from outside
-    my $Result = Win32::Service::StopService( '', $Service );
-
-    # sleep to let service stop successfully
-    sleep 2;
-
-    if ($Result) {
-        exit 0;
-    }
-    exit 1;
+    my $ExitCode = _Stop( Force => $Force );
+    exit $ExitCode;
 }
 
 # check if a stop service request is sent (this should be called by SCM)
 elsif ( $Opts{a} && $Opts{a} eq "servicestop" ) {
 
     # stop the scheduler service
-    _stop();
+    _ServiceStop();
 }
 
 # check if a status request is sent
 elsif ( $Opts{a} && $Opts{a} eq "status" ) {
 
-    # query scheduler status
-    _status();
+    my $ProcessID = _Status();
+
+    # no process ID means that is not running
+    if ( !$ProcessID ) {
+        print "Not Running!\n";
+    }
+
+    # the process ID is not -1
+    elsif ( IsPositiveInteger($ProcessID) ) {
+        print "Running $ProcessID\n";
+    }
+
+    # a process ID of -1, means that is not running but still registered
+    else {
+        print
+            "Not Running, but PID is still registered! Use '-a stop --force' to unregister "
+            . "the PID from the database.\n";
+    }
+
+    # set the correct exit code
+    my $ExitCode = IsPositiveInteger($ProcessID) ? 0 : 1;
+    exit $ExitCode;
 }
 
 # check if a reload request is sent
 elsif ( $Opts{a} && $Opts{a} eq "reload" ) {
 
-    # create common objects
-    my %CommonObject = _CommonObjects();
+    # get PID object
+    my $PIDObject = $Kernel::OM->Get('Kernel::System::PID');
 
     # get the process ID
-    my %PID = $CommonObject{PIDObject}->PIDGet(
-        Name => 'otrs.Scheduler',
-    );
+    my %SchedulerPID = $PIDObject->PIDGet( Name => $PIDName );
 
     # no process ID means that is not running
-    if ( !%PID ) {
-        print "Can't get OTRS Scheduler status, it is not running!\n";
+    if ( !%SchedulerPID ) {
+        print "Can't get OTRS Scheduler status because it is not running!\n";
         exit 1;
     }
 
     # log daemon reload request
-    $CommonObject{LogObject}->Log(
+    $Kernel::OM->Get('Kernel::System::Log')->Log(
         Priority => 'notice',
-        Message  => "Scheduler Service reload request: PID $PID{PID}",
+        Message  => "Scheduler Service reload request: PID $SchedulerPID{PID}",
     );
 
     # stop the scheduler service (same as "stop" in service control manger)
@@ -168,7 +166,10 @@ elsif ( $Opts{a} && $Opts{a} eq "reload" ) {
     sleep 2;
 
     # delete process ID lock
-    my $PIDDelSuccess = $CommonObject{PIDObject}->PIDDelete( Name => $PID{Name} );
+    my $PIDDelSuccess = $PIDObject->PIDDelete(
+        Name  => $SchedulerPID{Name},
+        Force => 1,
+    );
 
     # start the scheduler service (same as "play" in service control manager)
     # can't use Win32::Daemon because it is called from outside
@@ -177,11 +178,41 @@ elsif ( $Opts{a} && $Opts{a} eq "reload" ) {
     exit 0;
 }
 
-# check if a start request is sent
+# check if start request is sent
 elsif ( $Opts{a} && $Opts{a} eq "start" ) {
 
-    # create common objects
-    my %CommonObject = _CommonObjects();
+    my $Force = $Opts{f} ? 1 : '';
+
+    my $ExitCode = _Start( Force => $Force );
+    exit $ExitCode;
+}
+
+# check if a service start request is sent (This is usually called by SCM)
+elsif ( $Opts{a} && $Opts{a} eq "servicestart" ) {
+
+    # start the scheduler process
+    _ServiceStart();
+}
+
+# invalid option, show help
+else {
+    _Help();
+    exit 1;
+}
+
+exit 1;
+
+# Internal
+sub _Help {
+    print "otrs.Scheduler4win.pl - OTRS Scheduler Daemon\n";
+    print "Copyright (C) 2001-2018 OTRS AG, https://otrs.com/\n";
+    print "Usage: otrs.Scheduler4win.pl -a <ACTION> (start|stop|status|reload) [-f force]\n";
+    print "       otrs.Scheduler4win.pl -w 1 (Watchdog mode)\n";
+    return 1;
+}
+
+sub _Start {
+    my %Param = @_;
 
     # get current service status
     # can't use Win32::Daemon because it is called from outside
@@ -194,108 +225,99 @@ elsif ( $Opts{a} && $Opts{a} eq "start" ) {
     }
 
     # check if PID is already there
-    my %PID = $CommonObject{PIDObject}->PIDGet( Name => 'otrs.Scheduler' );
+    my %SchedulerPID = $Kernel::OM->Get('Kernel::System::PID')->PIDGet( Name => $PIDName );
 
-    if (%PID) {
+    if (%SchedulerPID) {
 
         # get the PID update time
         my $PIDUpdateTime =
-            $CommonObject{ConfigObject}->Get('Scheduler::PIDUpdateTime') || 60.0;
+            $Kernel::OM->Get('Kernel::Config')->Get('Scheduler::PIDUpdateTime') || 60.0;
 
         # get current time
         my $Time = time();
 
         # calculate time difference
-        my $DeltaTime = $Time - $PID{Changed};
+        my $DeltaTime = $Time - $SchedulerPID{Changed};
 
-        # remove PID if changed time is grater than
+        # remove PID if changed time is greater than
         if ( $DeltaTime > $PIDUpdateTime ) {
 
             # _AutoStop returns an exit code for the OS, we need the opposit value
             my $PIDDeleteSuccess = !_AutoStop(
-                Message => 'NOTICE: otrs.Shceduler4win.pl is registered on the DB, but the '
+                Message => 'NOTICE: otrs.Scheduler4win.pl is registered in the DB, but the '
                     . 'registry has not been updated in ' . $DeltaTime . ' seconds!. '
-                    . 'The register will be deleted so Scheduler can start again without '
+                    . 'The registration will be deleted so the Scheduler can start again without '
                     . 'forcing',
                 DeletePID => 1,
             );
 
             if ($PIDDeleteSuccess) {
-                %PID = ();
+                %SchedulerPID = ();
             }
         }
     }
 
-    # check for force to start option
-    if (%PID) {
+    my $ExitCode;
 
-        if ( !$Opts{f} ) {
+    # check for force to start option
+    if (%SchedulerPID) {
+        if ( !$Param{Force} ) {
             print
-                "NOTICE: otrs.Scheduler4win.pl is already running (use '-force' if you want to start it\n";
-            print "forced)!\n";
+                "NOTICE: otrs.Scheduler4win.pl is already running (use '-f' if you want to start it forced)!\n";
 
             # log daemon already running
-            $CommonObject{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message =>
-                    "Scheduler Service tries to start but found an already running service!\n",
+                    "Scheduler Service tried to start but found an already running service!\n",
             );
-            exit 1;
+            $ExitCode = 1;
+            return $ExitCode;
         }
-        elsif ( $Opts{f} ) {
-            print
-                "NOTICE: otrs.Scheduler4win.pl was already running but is starting again (force was used)!\n";
 
-            # log daemon forced start
-            $CommonObject{LogObject}->Log(
-                Priority => 'notice',
-                Message  => "Scheduler Service is forced to start!",
-            );
-        }
+        print
+            "NOTICE: otrs.Scheduler4win.pl was already running but is starting again (force was used)!\n";
+
+        # log daemon forced start
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'notice',
+            Message  => "Scheduler Service is forced to start!",
+        );
     }
 
     # start the scheduler service (same as "play" in service control manager)
     # cant use Win32::Daemon because is called from outside
-    Win32::Service::StartService( '', $Service );
+    my $Result = Win32::Service::StartService( '', $Service );
+
+    $ExitCode = $Result ? 0 : 1;
+    return $ExitCode;
 }
 
-# check if a service start request is sent (This is usually called by SCM)
-elsif ( $Opts{a} && $Opts{a} eq "servicestart" ) {
+sub _ServiceStart {
 
-    # start the scheduler process
-    _start();
-}
+    # if start is forced, be sure to remove any PID from any host
+    my $Force = $Opts{f} ? 1 : '';
 
-# otherwise show help
-else {
-
-    # help option
-    _help();
-}
-
-# Internal
-sub _help {
-    print "otrs.Scheduler4win.pl <Revision $VERSION> - OTRS Scheduler Daemon\n";
-    print "Copyright (C) 2001-2014 OTRS AG, http://otrs.com/\n";
-    print "usage: otrs.Scheduler4win.pl -a <ACTION> (start|stop|status|reload) [-f force]\n";
-}
-
-sub _start {
-
-    # create common objects
-    my %CommonObject = _CommonObjects();
+    # get PID object
+    my $PIDObject = $Kernel::OM->Get('Kernel::System::PID');
 
     # create new PID on the Database
-    $CommonObject{PIDObject}->PIDCreate( Name => 'otrs.Scheduler' );
-
-    # get the process ID
-    my %PID = $CommonObject{PIDObject}->PIDGet(
-        Name => 'otrs.Scheduler',
+    $PIDObject->PIDCreate(
+        Name  => $PIDName,
+        Force => $Force,
     );
 
+    # get the process ID
+    my %SchedulerPID = $PIDObject->PIDGet(
+        Name => $PIDName,
+    );
+
+    # get PID object
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
     # get default log path from configuration
-    my $LogPath = $CommonObject{ConfigObject}->Get('Scheduler::LogPath')
-        || '<OTRS_CONFIG_Home>/var/log';
+    my $LogPath = $ConfigObject->Get('Scheduler::LogPath')
+        || $ConfigObject->Get('Home') . '/var/log';
 
     # set proper directory separators on windows
     $LogPath =~ s{/}{\\}g;
@@ -303,8 +325,12 @@ sub _start {
     # backup old log files
     my $FileStdOut = $LogPath . '\SchedulerOUT.log';
     my $FileStdErr = $LogPath . '\SchedulerERR.log';
+
+    # get PID object
+    my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+
     use File::Copy;
-    my $SystemTime = $CommonObject{TimeObject}->SystemTime();
+    my $SystemTime = $TimeObject->SystemTime();
     if ( -e $FileStdOut ) {
         move( "$FileStdOut", "$LogPath/SchedulerOUT-$SystemTime.log" );
     }
@@ -313,11 +339,13 @@ sub _start {
     }
 
     # delete old log files
-    my $DaysToKeep = $CommonObject{ConfigObject}->Get('Scheduler::Log::DaysToKeep') || 10;
-    my $DaysToKeepSystemTime
-        = $CommonObject{TimeObject}->SystemTime() - $DaysToKeep * 24 * 60 * 60;
+    my $DaysToKeep = $ConfigObject->Get('Scheduler::Log::DaysToKeep') || 10;
+    my $DaysToKeepSystemTime = $TimeObject->SystemTime() - $DaysToKeep * 24 * 60 * 60;
 
-    my @LogFiles = <$LogPath/*.log>;
+    my @LogFiles = glob("$LogPath/*.log");
+
+    # get log object
+    my $LogObject = $Kernel::OM->Get('Kernel::System::Log');
 
     LOGFILE:
     for my $LogFile (@LogFiles) {
@@ -332,7 +360,7 @@ sub _start {
         if ( unlink($LogFile) == 0 ) {
 
             # log old backup file cannot be deleted
-            $CommonObject{LogObject}->Log(
+            $LogObject->Log(
                 Priority => 'error',
                 Message  => "Scheduler could not delete old backup file $LogFile! $!",
             );
@@ -341,7 +369,7 @@ sub _start {
         else {
 
             # log old backup file deleted
-            $CommonObject{LogObject}->Log(
+            $LogObject->Log(
                 Priority => 'notice',
                 Message  => "Scheduler deleted old backup file $LogFile!",
             );
@@ -352,15 +380,15 @@ sub _start {
     my $AlreadyStarted;
 
     # sleep time between loop intervals in seconds
-    my $SleepTime = $CommonObject{ConfigObject}->Get('Scheduler::SleepTime') || 1;
+    my $SleepTime = $ConfigObject->Get('Scheduler::SleepTime') || 1;
 
-    my $RestartAfterSeconds = $CommonObject{ConfigObject}->Get('Scheduler::RestartAfterSeconds')
+    my $RestartAfterSeconds = $ConfigObject->Get('Scheduler::RestartAfterSeconds')
         || ( 60 * 60 * 24 );    # default 1 day
 
-    my $StartTime = $CommonObject{TimeObject}->SystemTime();
+    my $StartTime = $TimeObject->SystemTime();
 
     # get config checksum
-    my $InitConfigMD5 = $CommonObject{ConfigObject}->ConfigChecksum();
+    my $InitConfigMD5 = $ConfigObject->ConfigChecksum();
 
     # start the service
     Win32::Daemon::StartService();
@@ -370,82 +398,80 @@ sub _start {
     $State = Win32::Daemon::State();
 
     # Redirect STDOUT and STDERR to log file
-    open( STDOUT, ">$FileStdOut" );
-    open( STDERR, ">$FileStdErr" );
+    open( STDOUT, ">", $FileStdOut );    ## no critic
+    open( STDERR, ">", $FileStdErr );    ## no critic
 
     # main service loop
-    while ( SERVICE_STOPPED != $State ) {
+    while ( SERVICE_STOPPED() != $State ) {
 
         # check if service is in start pending state
-        if ( SERVICE_START_PENDING == $State ) {
+        if ( SERVICE_START_PENDING() == $State ) {
 
             # Log service start-up
-            $CommonObject{LogObject}->Log(
+            $LogObject->Log(
                 Priority => 'notice',
                 Message  => "Scheduler Service is starting...!",
             );
 
             # set running state
-            Win32::Daemon::State(SERVICE_RUNNING);
+            Win32::Daemon::State( SERVICE_RUNNING() );
         }
 
         # check if service is in pause pending state
-        elsif ( SERVICE_PAUSE_PENDING == $State ) {
+        elsif ( SERVICE_PAUSE_PENDING() == $State ) {
 
             # Log service pause
-            $CommonObject{LogObject}->Log(
+            $LogObject->Log(
                 Priority => 'notice',
                 Message  => "Scheduler Service is pausing...!",
             );
 
             # set paused state
-            Win32::Daemon::State(SERVICE_PAUSED);
+            Win32::Daemon::State( SERVICE_PAUSED() );
         }
 
         # check if service is in continue pending state
-        elsif ( SERVICE_CONTINUE_PENDING == $State ) {
+        elsif ( SERVICE_CONTINUE_PENDING() == $State ) {
 
             # Log service resume
-            $CommonObject{LogObject}->Log(
+            $LogObject->Log(
                 Priority => 'notice',
                 Message  => "Scheduler Service resuming...!",
             );
 
             # set running state
-            Win32::Daemon::State(SERVICE_RUNNING);
+            Win32::Daemon::State( SERVICE_RUNNING() );
         }
 
         # check if service is in stop pending state
-        elsif ( SERVICE_STOP_PENDING == $State ) {
+        elsif ( SERVICE_STOP_PENDING() == $State ) {
 
             # Log service stop
-            $CommonObject{LogObject}->Log(
+            $LogObject->Log(
                 Priority => 'notice',
                 Message  => "Scheduler Service is stopping...!",
             );
 
             # set stop state
-            Win32::Daemon::State(SERVICE_STOPPED);
+            Win32::Daemon::State( SERVICE_STOPPED() );
         }
 
         # check if service is running
-        elsif ( SERVICE_RUNNING == $State ) {
+        elsif ( SERVICE_RUNNING() == $State ) {
 
             if ( !$AlreadyStarted ) {
-                $CommonObject{LogObject}->Log(
+                $LogObject->Log(
                     Priority => 'notice',
-                    Message  => "Scheduler Service start! PID $PID{PID}",
+                    Message  => "Scheduler Service start! PID $SchedulerPID{PID}",
                 );
                 $AlreadyStarted = 1;
             }
 
             # get the process ID
-            my %PID = $CommonObject{PIDObject}->PIDGet(
-                Name => 'otrs.Scheduler',
-            );
+            my %SchedulerPID = $PIDObject->PIDGet( Name => $PIDName );
 
             # check if process ID was deleted from DB
-            if ( !%PID ) {
+            if ( !%SchedulerPID ) {
                 my $ExitCode = _AutoStop(
                     Message => "Process could not be found in the process table!\n"
                         . "Scheduler is stopping...!\n",
@@ -453,9 +479,9 @@ sub _start {
                 return $ExitCode;
             }
 
-            # check if Framework.xml file exists, otherwise quits because the otrs installation
+            # check if Framework.xml file exists, otherwise quit because the otrs installation
             # might not be OK. for example UnitTest machines during change scenario process
-            my $Home                = $CommonObject{ConfigObject}->Get('Home');
+            my $Home                = $ConfigObject->Get('Home');
             my $FrameworkConfigFile = $Home . '/Kernel/Config/Files/Framework.xml';
 
             # convert FrameworkConfigFile path to windows format
@@ -472,7 +498,7 @@ sub _start {
             }
 
             # get config checksum
-            my $CurrConfigMD5 = $CommonObject{ConfigObject}->ConfigChecksum();
+            my $CurrConfigMD5 = $ConfigObject->ConfigChecksum();
 
             # check if checksum changed and restart
             if ( $InitConfigMD5 ne $CurrConfigMD5 ) {
@@ -485,16 +511,16 @@ sub _start {
             }
 
             # Call Scheduler
-            my $SchedulerObject = Kernel::Scheduler->new(%CommonObject);
+            my $SchedulerObject = $Kernel::OM->Get('Kernel::System::Scheduler');
             $SchedulerObject->Run();
 
-            my $CurrentTime = $CommonObject{TimeObject}->SystemTime();
+            my $CurrentTime = $TimeObject->SystemTime();
 
             # The Scheduler needs to be restarted from time to time because
             #   of memory leaks in some external perl modules.
             if ( ( $CurrentTime - $StartTime ) > $RestartAfterSeconds ) {
                 my $ExitCode = _AutoRestart(
-                    Message => "Scheduler Service restarts itself (PID $PID{PID})."
+                    Message => "Scheduler Service restarts itself (PID $SchedulerPID{PID})."
                 );
                 exit $ExitCode;
             }
@@ -507,46 +533,87 @@ sub _start {
     }
 
     # stop the service
-    _stop();
+    _ServiceStop();
+    return 1;
 }
 
-sub _stop {
+sub _Stop {
+    my %Param = @_;
 
-    # create common objects
-    my %CommonObject = _CommonObjects();
+    my $ExitCode;
+
+    if ( $Param{Force} ) {
+
+        # delete process ID lock
+        my $PIDDelSuccess = $Kernel::OM->Get('Kernel::System::PID')->PIDDelete(
+            Name  => $PIDName,
+            Force => 1,
+        );
+    }
+    else {
+
+        # get the process ID
+        my %SchedulerPID = $Kernel::OM->Get('Kernel::System::PID')->PIDGet( Name => $PIDName );
+
+        # no process ID means that is not running
+        if ( !%SchedulerPID ) {
+            print "Can't stop OTRS Scheduler because is not running!\n";
+            $ExitCode = 1;
+            return $ExitCode;
+        }
+    }
+
+    # stop the scheduler service (same as "stop"" in service control manger)
+    # cant use Win32::Daemon because is called from outside
+    my $Result = Win32::Service::StopService( '', $Service );
+
+    # sleep to let service stop successfully
+    sleep 2;
+
+    $ExitCode = $Result ? 0 : 1;
+    return $ExitCode;
+}
+
+sub _ServiceStop {
+
+    # get database object
+    my $PIDObject = $Kernel::OM->Get('Kernel::System::PID');
 
     # get the process ID
-    my %PID = $CommonObject{PIDObject}->PIDGet(
-        Name => 'otrs.Scheduler',
-    );
+    my %SchedulerPID = $PIDObject->PIDGet( Name => $PIDName );
 
     # stop the service (this can be called because is part of the main loop)
     Win32::Daemon::StopService();
 
     # delete process ID lock
-    my $PIDDelSuccess = $CommonObject{PIDObject}->PIDDelete( Name => 'otrs.Scheduler' );
+    # do not delete processes from other hosts at this time
+    my $PIDDelSuccess = $PIDObject->PIDDelete(
+        Name => $SchedulerPID{Name},
+    );
 
     sleep 2;
 
     # log daemon stop
     if ( !$PIDDelSuccess ) {
-        $CommonObject{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message  => "Process could not be deleted from process table! PID $PID{PID}",
+            Message  => "Process could not be deleted from process table! PID $SchedulerPID{PID}",
         );
         exit 1;
     }
-    $CommonObject{LogObject}->Log(
+    $Kernel::OM->Get('Kernel::System::Log')->Log(
         Priority => 'notice',
-        Message  => "Scheduler Service stop! PID $PID{PID}",
+        Message  => "Scheduler Service stop! PID $SchedulerPID{PID}",
     );
     exit 0;
 }
 
-sub _status {
+sub _Status {
+    my %Param = @_;
 
     # Windows service status table
     # 5 => 'The service continue is pending.',
+
     # 6 => 'The service pause is pending.',
     # 7 => 'The service is paused.',
     # 4 => 'The service is running.',
@@ -554,24 +621,18 @@ sub _status {
     # 3 => 'The service is stopping.',
     # 1 => 'The service is not running.',
 
-    # create common objects
-    my %CommonObject = _CommonObjects();
-
     # get the process ID
-    my %PID = $CommonObject{PIDObject}->PIDGet(
-        Name => 'otrs.Scheduler',
-    );
+    my %SchedulerPID = $Kernel::OM->Get('Kernel::System::PID')->PIDGet( Name => $PIDName );
 
     # no process ID means that is not running
-    if ( !%PID ) {
-        print "Not running!\n";
-        exit 1;
+    if ( !%SchedulerPID ) {
+        return 0;
     }
 
     # log daemon stop
-    $CommonObject{LogObject}->Log(
+    $Kernel::OM->Get('Kernel::System::Log')->Log(
         Priority => 'notice',
-        Message  => "Scheduler Service status request! PID $PID{PID}",
+        Message  => "Scheduler Service status request! PID $SchedulerPID{PID}",
     );
 
     # this call is from outside, can't use Win32::Daemon
@@ -583,60 +644,44 @@ sub _status {
         sleep 1;
     }
 
-    # check if service is running (state 4)
     if ( $ServiceStatus->{CurrentState} eq 4 ) {
-        print "Running $PID{PID}\n"
-    }
-    else {
-        print
-            "Not running, but PID still registered! Use '-a stop -force' to unregister the PID from the database.\n";
-        exit 1;
+        return $SchedulerPID{PID};
     }
 
-    exit 0;
-}
-
-sub _CommonObjects {
-    my %CommonObject;
-    $CommonObject{ConfigObject} = Kernel::Config->new();
-    $CommonObject{EncodeObject} = Kernel::System::Encode->new(%CommonObject);
-    $CommonObject{LogObject}    = Kernel::System::Log->new(
-        LogPrefix => 'OTRS-otrs.Scheduler',
-        %CommonObject,
-    );
-    $CommonObject{MainObject} = Kernel::System::Main->new(%CommonObject);
-    $CommonObject{TimeObject} = Kernel::System::Time->new(%CommonObject);
-    $CommonObject{DBObject}   = Kernel::System::DB->new(%CommonObject);
-    $CommonObject{PIDObject}  = Kernel::System::PID->new(%CommonObject);
-    return %CommonObject;
+    # otherwise return -1, this means that the process is registed but it not running
+    return -1;
 }
 
 sub _AutoRestart {
-    my %Param = @_;
+    my (%Param) = @_;
 
-    # create common objects
-    my %CommonObject = _CommonObjects();
+    # get PID object
+    my $PIDObject = $Kernel::OM->Get('Kernel::System::PID');
 
     # get the process ID
-    my %PID = $CommonObject{PIDObject}->PIDGet(
-        Name => 'otrs.Scheduler',
-    );
+    my %SchedulerPID = $PIDObject->PIDGet( Name => $PIDName );
+
+    # get log object
+    my $LogObject = $Kernel::OM->Get('Kernel::System::Log');
 
     # Log daemon start-up
-    $CommonObject{LogObject}->Log(
+    $LogObject->Log(
         Priority => 'notice',
-        Message => $Param{Message} || 'Unknown reason to restart',
+        Message  => $Param{Message} || 'Unknown reason to restart',
     );
 
     # delete process ID lock
-    my $PIDDelSuccess = $CommonObject{PIDObject}->PIDDelete( Name => $PID{Name} );
+    my $PIDDelSuccess = $PIDObject->PIDDelete(
+        Name  => $SchedulerPID{Name},
+        Force => 1,
+    );
 
     my $ExitCode;
     if ( !$PIDDelSuccess ) {
-        $CommonObject{LogObject}->Log(
+        $LogObject->Log(
             Priority => 'error',
             Message =>
-                "Could not remove Scheduler PID $PID{PID} from database to prepare Scheduler restart, exiting.",
+                "Could not remove Scheduler PID $SchedulerPID{PID} from database to prepare Scheduler restart, exiting.",
         );
         $ExitCode = 1;
         return $ExitCode;
@@ -647,13 +692,13 @@ sub _AutoRestart {
     Win32::Daemon::StopService();
 
     # log service stopping
-    $CommonObject{LogObject}->Log(
+    $LogObject->Log(
         Priority => 'notice',
         Message  => "Scheduler service is stopping due a restart.",
     );
 
     # get the scheduler information to restart
-    my $Home      = $CommonObject{ConfigObject}->Get('Home');
+    my $Home      = $Kernel::OM->Get('Kernel::Config')->Get('Home');
     my $Scheduler = $Home . '/bin/otrs.Scheduler4win.pl';
 
     # convert Scheduler path to windows format
@@ -662,10 +707,10 @@ sub _AutoRestart {
     # create a new scheduler instance
     # this process could take more than 30 seconds be aware of that!
     # needs a separate process
-    my $Result = system("\"$^X\" \"$Scheduler\" -a start");
+    my $StartExitCode = system("\"$^X\" \"$Scheduler\" -a start");
 
-    if ( !$Result ) {
-        $CommonObject{LogObject}->Log(
+    if ($StartExitCode) {
+        $LogObject->Log(
             Priority => 'error',
             Message  => "Could not start-up new Scheduler instance.",
         );
@@ -679,15 +724,12 @@ sub _AutoRestart {
 }
 
 sub _AutoStop {
-    my %Param = @_;
-
-    # create common objects
-    my %CommonObject = _CommonObjects();
+    my (%Param) = @_;
 
     if ( $Param{Message} ) {
 
         # log error
-        $CommonObject{LogObject}->Log(
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
             Message  => $Param{Message},
         );
@@ -696,19 +738,22 @@ sub _AutoStop {
     my $ExitCode;
     if ( $Param{DeletePID} ) {
 
+        # get PID object
+        my $PIDObject = $Kernel::OM->Get('Kernel::System::PID');
+
         # get the process ID
-        my %PID = $CommonObject{PIDObject}->PIDGet(
-            Name => 'otrs.Scheduler',
-        );
+        my %SchedulerPID = $PIDObject->PIDGet( Name => $PIDName );
 
         # delete process ID lock
-        my $PIDDelSuccess = $CommonObject{PIDObject}->PIDDelete( Name => $PID{Name} );
+        # scheduler should not delete PIDs from other hots at this point
+        my $PIDDelSuccess = $PIDObject->PIDDelete( Name => $SchedulerPID{Name} );
 
         # log daemon stop
         if ( !$PIDDelSuccess ) {
-            $CommonObject{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
-                Message  => "Process could not be deleted from process table! PID $PID{PID}",
+                Message =>
+                    "Process could not be deleted from process table! PID $SchedulerPID{PID}",
             );
             $ExitCode = 1;
 
@@ -729,3 +774,53 @@ sub _AutoStop {
     Win32::Daemon::StopService();
     return $ExitCode;
 }
+
+sub _WatchDog {
+    my %Param = @_;
+
+    local $Kernel::OM = Kernel::System::ObjectManager->new(
+        'Kernel::System::Log' => {
+            LogPrefix => 'OTRS-otrs.Scheduler-Watchdog',
+        },
+        'Kernel::System::DB' => {
+            AutoConnectNo => 1,
+        },
+    );
+
+    my $ExitCode = 0;
+
+    # check if OTRS can connect to the DB
+    if ( !$Kernel::OM->Get('Kernel::System::DB')->Connect() ) {
+        $Kernel::OM->Get('Kernel::System::Log')->Log(
+            Priority => 'notice',
+            Message  => "Database is not ready!",
+        );
+        return $ExitCode;
+    }
+
+    my $ProcessID = _Status();
+
+    if ( !$ProcessID ) {
+        $ExitCode = _Start();
+    }
+    elsif ( $ProcessID == -1 ) {
+        _Stop( Force => 1 );
+
+        my $ProcessID = _Status();
+        if ( !$ProcessID ) {
+            $ExitCode = _Start();
+        }
+        else {
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
+                Priority => 'error',
+                Message =>
+                    'Scheduler was forced to stop but it is still registered, can not continue',
+            );
+            $ExitCode = 1;
+        }
+    }
+
+    # if there is a $PID and it is not -1 then it means that the scheduler is running, nothing to
+    #   do here
+    return $ExitCode;
+}
